import argparse
import glob
import os
import platform
import re
import subprocess
import shutil
import sys

############################## INPUT PARAMETERS #############################

# Give a name for the project
ProjectName = "Armchair_1"

# Should the renderers be with thumbnail size. Set to True if thumbnails are desired
ThumbnailsOnly = False

# Size of the thumbnails. Used if ThumbnailsOnly is set to True
ThumbnailWidth = '100'
ThumbnailHeight = '100'

#############################################################################

argumentsParser = argparse.ArgumentParser()
argumentsParser.register('type', 'bool', (lambda x: x.lower() in ("yes", "true", "t", "1")))
argumentsParser.add_argument('--project', nargs='?', default=ProjectName)
argumentsParser.add_argument('--thumbnails', nargs='?', type=bool, default=ThumbnailsOnly)
args = argumentsParser.parse_args()

ProjectName = args.project
ThumbnailsOnly = args.thumbnails

systemName = platform.system()

# Configure the path to the V-Ray Cloud Client:
vcloud = ''
if systemName == 'Windows':
    vcloud = os.path.join(os.getenv('APPDATA'), r'Chaos Group\V-Ray\V-Ray Cloud Client\vcloud.exe')
elif systemName == 'Darwin':
    vcloud = '/Applications/ChaosGroup/V-Ray/V-Ray Cloud Client/vcloud'
elif systemName == 'Linux':
    vcloud = os.path.join(os.getenv('HOME'), r'.ChaosGroup/vcloud/client/vcloud.bin')
else:
    print('Unsupported operating system {0}'.format(systemName))
    exit()

# Base path is relative to the directory in which the script is placed:
basePath = os.path.dirname(os.path.abspath(__file__))
scenesPath = os.path.join(basePath, 'Scenes')
rendersPath = os.path.join(basePath, 'Renders')

if ThumbnailsOnly:
    rendersPath = os.path.join(basePath, 'Renders_Thumbnails')

if not os.path.isdir(rendersPath):
    os.makedirs(rendersPath, 0o755)

# Create the project:
try:
    createProjectCommand = [
        vcloud,
        'project',
        'create',
        '--name={0}'.format(ProjectName),
    ]

    print(' '.join(createProjectCommand))
    print(subprocess.check_output(createProjectCommand, stderr=subprocess.STDOUT).decode('utf-8'))
except subprocess.CalledProcessError as e:
    print('Error while creating project:')
    print(e.output.decode('utf-8'))

jobIDs=[]
jobList=[]

# Prepare the jobs to submit:
for sceneName in os.listdir(scenesPath):
    # Remove the .vrscene:
    jobName, _ = os.path.splitext(sceneName)

    if ThumbnailsOnly:
        jobName = jobName + '_Thumbnail'

    # Add a tuple for the job (jobName, fullScenePath):
    jobList.append((jobName, os.path.join(scenesPath, sceneName)))

# Submit the prepared jobs in the created project:
for jobName, scenePath in jobList:
    submitCommand = [
        vcloud,
        'job',
        'submit',
        '--name={0}'.format(jobName),
        '--sceneFile={0}'.format(scenePath),
        '--project={0}'.format(ProjectName),
        '--ignoreWarnings',
        '--no-progress',
    ]

    if ThumbnailsOnly and ThumbnailWidth and ThumbnailHeight:
        submitCommand.append('--width={0}'.format(ThumbnailWidth))
        submitCommand.append('--height={0}'.format(ThumbnailHeight))

    try:
        print(' '.join(submitCommand))
        submitCommandOutput = subprocess.check_output(submitCommand, stderr=subprocess.STDOUT)
        print(submitCommandOutput.decode('utf-8'))

        match = re.search(r'Job ID: (?P<jobID>.*)', submitCommandOutput.decode('utf-8'))
        if match:
            jobIDs.append(match.group('jobID')) # Save the job ID for later
    except subprocess.CalledProcessError as e:
        print('Error while submitting scene {0}:'.format(scenePath))
        print(e.output.decode('utf-8'))

print('''
______________________________________________________________________

Follow the progress in V-Ray Cloud platform:
https://vray.cloud/projects/{project}

After all of the render jobs are complete, press "Enter" to download.
______________________________________________________________________

'''.format(project=ProjectName))
wait=sys.stdin.readline()

# Download the final images
for jobID in jobIDs:
    try:
        downloadCommand = [
            vcloud,
            'job',
            'download',
            '--id={0}'.format(jobID),
            '--output={0}'.format(rendersPath),
            '--no-progress',
        ]
        print(' '.join(downloadCommand))
        print(subprocess.check_output(downloadCommand, stderr=subprocess.STDOUT).decode('utf-8'))
    except subprocess.CalledProcessError as e:
        print('Error while downloading job {0}:'.format(jobID))
        print(e.output.decode('utf-8'))

########################________Making new folder_________##########################
newFolder = os.path.join(rendersPath, 'Beauty')

if ThumbnailsOnly:
    newFolder = os.path.join(rendersPath, 'Thumbnails')

if not os.path.isdir(newFolder):
    os.makedirs(newFolder, 0o755)

for jobName, _ in jobList:
    sdir = os.path.join(rendersPath, jobName)

    for jpeg in glob.iglob(os.path.join(sdir, "*.jpg")):
        shutil.copy2(jpeg, newFolder)

print('''
______________________________________________________________________

The images were saved in {rendersPath}
______________________________________________________________________

Press "Enter" to escape.
'''.format(rendersPath=rendersPath))

wait=sys.stdin.readline()